#!/usr/bin/env python

class CassandraLauncher:
    def __init__(self):
        import cassandralauncher
        cassandralauncher.run()

class ClusterLauncher:
    def __init__(self):
        import clusterlauncher
        clusterlauncher.run()
